# Hibernate 4 注释配置

> 原文： [https://javabeginnerstutorial.com/hibernate/hibernate-4-annotations-configuration/](https://javabeginnerstutorial.com/hibernate/hibernate-4-annotations-configuration/)

在最后的介绍性文章中，我提到了所谓的“XML-Hell”，它是 XML 在 Hibernate 配置中的大量使用。 在本文中，我将介绍基于注释的配置，您可以在其中使用 Hibernate 在实体上的注释来减少所需的 XML 数量。

## 为什么注释很好

正如您将在示例代码中看到的那样：使用注释，您可以在实体类定义本身中看到属性映射是什么，因此您无需查找正确的`.hbm.xml`文件即可查找 映射定义。

而且有一个很好的副作用：您只需要修改实体。 例如，如果要向`Book`类添加新的 Date 字段，则还需要在`Book.hbm.xml`文件中添加映射。 使用注释，这只是 Java 类中的更改。 您将在本文后面的示例中找到有关使用日期的示例。

### 一个例子

在此示例中，我将继续使用第一篇文章中介绍的 Book 实体。 现在，我将转换 POJO 以使用批注而不是 XML 配置-在本系列的后续部分中，我将继续介绍。 这是因为注释比 XML 更易于阅读和查找。

#### 实体

对于实体，我需要添加一些注释以表示与 XML 文件中相同的映射。

第一个是告诉 Hibernate `Book`类是一个实体。 我们可以通过`@Entity`批注来实现。 否则，Hibernate 将在启动时引发异常，并显示错误消息“`hibernate_example.Book`”是未知实体。

下一个是表定义。 我将书添加到`BOOKS`表中。 如果没有明确的表定义，Hibernate 会将实体添加到 `BOOK`表中。 通常，我可以说没有表定义，实体将保存在与实体类相同名称的表中。 要命名表，我必须在课程上使用`@Table`。 该注释具有名为*名称*的参数。 此名称定义表的名称-因此我添加了此属性并将其命名为 `BOOKS`。

最后，我将 ISBN 定义为实体的 **ID**。 这是通过`@ID`注释完成的。

让我们看看整个修改后的实体：

```java
@Entity
 @Table(name = "BOOKS")
 public class Book {

    @Id
    private String isbn;
    private String title;
    private String author;
    // other parts stayed the same so I omit them here
 }
```

#### 配置

配置文件需要进行一次修改：我必须在最后将*映射*标记从使用**资源**更改为**类**，并提供映射类的限定名称 ：

```java
<hibernate-configuration>
    <session-factory>

        <!-- Database connection settings -->
        <property name="connection.driver_class">org.h2.Driver</property>
        <property name="connection.url">jdbc:h2:mem:db1;DB_CLOSE_DELAY=-1;MVCC=TRUE</property>
        <property name="connection.username">sa</property>
        <property name="connection.password"/>

        <!-- JDBC connection pool (use the built-in) -->
        <property name="connection.pool_size">1</property>

        <!-- SQL dialect -->
        <property name="dialect">org.hibernate.dialect.H2Dialect</property>

        <!-- Disable the second-level cache  -->
        <property name="cache.provider_class">org.hibernate.cache.internal.NoCacheProvider</property>

        <!-- Echo all executed SQL to stdout -->
        <property name="show_sql">true</property>

        <!-- Drop and re-create the database schema on startup -->
        <property name="hbm2ddl.auto">create</property>

		<!-- The mapping information of entities -->
        <mapping class="hibernate_example.Book"/>

    </session-factory>
</hibernate-configuration>
```

#### 映射文件

可以删除映射文件，因为 Hibernate 不再使用它了。

## 扩展实体

前面我提到过，添加日期字段有点复杂，因为映射日期并不是那么简单的动作。 为此，我需要在字段上使用`@Temporal`批注，以告知 Hibernate 我要将日期存储在数据库中，并且当我读取数据库时希望返回日期 信息。 `javax.persistence.TemporalType`告诉我要存储哪种信息。 现在是日期。

```java
@Temporal(TemporalType.DATE)
 private Date published;
```

对于其他正在阅读您的代码的开发人员来说，这很清楚，已将字段`published`映射为应用和数据库之间的 Date 类型。

对于 XML 配置，我将不得不使用以下配置：

```java
<property name="date" type="date" />
```

## 运行应用

该应用也可以像上一篇文章中那样运行，并且结果相同。

## 结论

Hibernate 可以利用一组标准注释来消除映射 XML 文件的需要。 通过使用标准`javax.persistence`批注，您可以从 Hibernate 切换到另一个实现标准接口的 ORM 解决方案。

但是，如今有些开发人员正在谈论“注释地狱”，因为您几乎可以使用注释配置任何内容，有时这会使应用的可读性变差。

在下一篇文章中，我将向您展示如何使用 Hibernate 建模实体之间的关系。

#### 代码下载

您可以从 Github [此处](https://github.com/JBTAdmin/Hibernate)下载特定章节的代码。

